# Information on VB commands from:
# https://github.com/SengerM/TeledyneLeCroyPy/blob/main/TeledyneLeCroyPy/__init__.py
# https://ohwr.org/project/wr-abscal-el-auto/tree/master/lib

import pyvisa as visa

class WAVERUNNER_8104_API:
    rm = visa.ResourceManager()
    vicp_addr = ""

    def __init__(self):
        pass

    def connect(self, address):
        vicp_addr = "VICP::" + address + "::INSTR"
        try:
            self.scope = self.rm.open_resource(vicp_addr)
            self.scope.timeout = 10000 # ms
            return True
        except:
            return False
        
    def get_info(self):
        return self.scope.query("*IDN?").split("*IDN ")[1].rstrip()

    def clear_measures(self):
        self.scope.write("VBS? 'app.Measure.clearsweeps'")

    def get_mean(self, meas_nb):
        return self.scope.query("VBS? 'return=app.Measure.P{}.mean.Result.Value'".format(meas_nb)).split("VBS ")[1].rstrip()
    
    def get_min(self, meas_nb):
        return self.scope.query("VBS? 'return=app.Measure.P{}.min.Result.Value'".format(meas_nb)).split("VBS ")[1].rstrip()

    def get_max(self, meas_nb):
        return self.scope.query("VBS? 'return=app.Measure.P{}.max.Result.Value'".format(meas_nb)).split("VBS ")[1].rstrip()
    
    def get_peak2peak(self, meas_nb):
        return self.scope.query("VBS? 'return=app.Measure.P{}.pkpk.Result.Value'".format(meas_nb)).split("VBS ")[1].rstrip()

    def get_sdev(self, meas_nb):
        return self.scope.query("VBS? 'return=app.Measure.P{}.sdev.Result.Value'".format(meas_nb)).split("VBS ")[1].rstrip()

    def get_nsamples(self, meas_nb):
        return self.scope.query("VBS? 'return=app.Measure.P{}.histo.Result.Sweeps'".format(meas_nb)).split("VBS ")[1].rstrip()